@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Igalia S.L.
+ * Copyright (C) 2010, 2011 Igalia S.L.
  *
  * Contact: Iago Toral Quiroga <itoral@igalia.com>
  *
@@ -73,14 +73,6 @@ GRL_LOG_DOMAIN_STATIC(podcasts_log_domain);
   "GROUP BY p.id "					\
   "LIMIT %u OFFSET %u"
 
-#define GRL_SQL_GET_PODCASTS_BY_TEXT				\
-  "SELECT p.*, count(s.podcast <> '') "				\
-  "FROM podcasts p LEFT OUTER JOIN streams s "			\
-  "  ON p.id = s.podcast "					\
-  "WHERE p.title LIKE '%%%s%%' OR p.desc LIKE '%%%s%%' "	\
-  "GROUP BY p.id "						\
-  "LIMIT %u OFFSET %u"
-
 #define GRL_SQL_GET_PODCASTS_BY_QUERY				\
   "SELECT p.*, count(s.podcast <> '') "				\
   "FROM podcasts p LEFT OUTER JOIN streams s "			\
@@ -120,6 +112,18 @@ GRL_LOG_DOMAIN_STATIC(podcasts_log_domain);
   "WHERE podcast='%s' "                         \
   "LIMIT %u  OFFSET %u"
 
+#define GRL_SQL_GET_PODCAST_STREAMS_BY_TEXT                     \
+  "SELECT s.* "                                                 \
+  "FROM streams s LEFT OUTER JOIN podcasts p "			\
+  "  ON s.podcast = p.id "					\
+  "WHERE s.title LIKE '%%%s%%' OR s.desc LIKE '%%%s%%' "	\
+  "  OR p.title LIKE '%%%s%%' OR p.desc LIKE '%%%s%%' "         \
+  "LIMIT %u OFFSET %u"
+
+#define GRL_SQL_GET_PODCAST_STREAMS_ALL         \
+  "SELECT * FROM streams "                      \
+  "LIMIT %u OFFSET %u"
+
 #define GRL_SQL_GET_PODCAST_STREAM              \
   "SELECT * FROM streams "                      \
   "WHERE url='%s' "                             \
@@ -186,6 +190,7 @@ typedef struct {
 struct _GrlPodcastsPrivate {
   sqlite3 *db;
   GrlNetWc *wc;
+  gboolean notify_changes;
 };
 
 typedef struct {
@@ -230,6 +235,10 @@ static void grl_podcasts_source_store (GrlMediaSource *source,
                                        GrlMediaSourceStoreSpec *ss);
 static void grl_podcasts_source_remove (GrlMediaSource *source,
                                         GrlMediaSourceRemoveSpec *rs);
+static gboolean grl_podcasts_source_notify_change_start (GrlMediaSource *source,
+                                                         GError **error);
+static gboolean grl_podcasts_source_notify_change_stop (GrlMediaSource *source,
+                                                        GError **error);
 
 /* =================== Podcasts Plugin  =============== */
 
@@ -238,7 +247,7 @@ grl_podcasts_plugin_init (GrlPluginRegistry *registry,
                           const GrlPluginInfo *plugin,
                           GList *configs)
 {
-  GRL_LOG_DOMAIN_INIT (podcasts_log_domain, "pocasts");
+  GRL_LOG_DOMAIN_INIT (podcasts_log_domain, "podcasts");
 
   GRL_DEBUG ("podcasts_plugin_init");
 
@@ -282,6 +291,8 @@ grl_podcasts_source_class_init (GrlPodcastsSourceClass * klass)
   source_class->metadata = grl_podcasts_source_metadata;
   source_class->store = grl_podcasts_source_store;
   source_class->remove = grl_podcasts_source_remove;
+  source_class->notify_change_start = grl_podcasts_source_notify_change_start;
+  source_class->notify_change_stop = grl_podcasts_source_notify_change_stop;
 
   metadata_class->supported_keys = grl_podcasts_source_supported_keys;
 
@@ -634,8 +645,22 @@ produce_podcast_contents_from_db (OperationSpec *os)
   GRL_DEBUG ("produce_podcast_contents_from_db");
 
   db = GRL_PODCASTS_SOURCE (os->source)->priv->db;
-  sql = g_strdup_printf (GRL_SQL_GET_PODCAST_STREAMS,
-			 os->media_id, os->count, os->skip);
+  /* Check if searching or browsing */
+  if (os->is_query) {
+    if (os->text) {
+      /* Search text */
+      sql = g_strdup_printf (GRL_SQL_GET_PODCAST_STREAMS_BY_TEXT,
+                             os->text, os->text, os->text, os->text,
+                             os->count, os->skip);
+    } else {
+      /* Return all */
+      sql = g_strdup_printf (GRL_SQL_GET_PODCAST_STREAMS_ALL,
+                             os->count, os->skip);
+    }
+  } else {
+    sql = g_strdup_printf (GRL_SQL_GET_PODCAST_STREAMS,
+                           os->media_id, os->count, os->skip);
+  }
   GRL_DEBUG ("%s", sql);
   r = sqlite3_prepare_v2 (db, sql, strlen (sql), &sql_stmt, NULL);
   g_free (sql);
@@ -712,7 +737,9 @@ remove_podcast_streams (sqlite3 *db, const gchar *podcast_id, GError **error)
 }
 
 static void
-remove_podcast (sqlite3 *db, const gchar *podcast_id, GError **error)
+remove_podcast (GrlPodcastsSource *podcasts_source,
+                const gchar *podcast_id,
+                GError **error)
 {
   gint r;
   gchar *sql_error;
@@ -720,27 +747,35 @@ remove_podcast (sqlite3 *db, const gchar *podcast_id, GError **error)
 
   GRL_DEBUG ("remove_podcast");
 
-  remove_podcast_streams (db, podcast_id, error);
+  remove_podcast_streams (podcasts_source->priv->db, podcast_id, error);
   if (*error) {
     return;
   }
 
   sql = g_strdup_printf (GRL_SQL_REMOVE_PODCAST, podcast_id);
   GRL_DEBUG ("%s", sql);
-  r = sqlite3_exec (db, sql, NULL, NULL, &sql_error);
+  r = sqlite3_exec (podcasts_source->priv->db, sql, NULL, NULL, &sql_error);
   g_free (sql);
 
   if (r != SQLITE_OK) {
     GRL_WARNING ("Failed to remove podcast '%s': %s", podcast_id, sql_error);
-    *error = g_error_new (GRL_CORE_ERROR,
-			  GRL_CORE_ERROR_REMOVE_FAILED,
-			  "Failed to remove podcast");
+    g_set_error_literal (error,
+                         GRL_CORE_ERROR,
+                         GRL_CORE_ERROR_REMOVE_FAILED,
+                         "Failed to remove podcast");
     sqlite3_free (sql_error);
+  } else if (podcasts_source->priv->notify_changes) {
+    grl_media_source_notify_change (GRL_MEDIA_SOURCE (podcasts_source),
+                                    NULL,
+                                    GRL_CONTENT_REMOVED,
+                                    TRUE);
   }
 }
 
 static void
-remove_stream (sqlite3 *db, const gchar *url, GError **error)
+remove_stream (GrlPodcastsSource *podcasts_source,
+               const gchar *url,
+               GError **error)
 {
   gint r;
   gchar *sql_error;
@@ -750,20 +785,28 @@ remove_stream (sqlite3 *db, const gchar *url, GError **error)
 
   sql = g_strdup_printf (GRL_SQL_REMOVE_STREAM, url);
   GRL_DEBUG ("%s", sql);
-  r = sqlite3_exec (db, sql, NULL, NULL, &sql_error);
+  r = sqlite3_exec (podcasts_source->priv->db, sql, NULL, NULL, &sql_error);
   g_free (sql);
 
   if (r != SQLITE_OK) {
     GRL_WARNING ("Failed to remove podcast stream '%s': %s", url, sql_error);
-    *error = g_error_new (GRL_CORE_ERROR,
-			  GRL_CORE_ERROR_REMOVE_FAILED,
-			  "Failed to remove podcast stream");
+    g_set_error_literal (error,
+                         GRL_CORE_ERROR,
+                         GRL_CORE_ERROR_REMOVE_FAILED,
+                         "Failed to remove podcast stream");
     sqlite3_free (sql_error);
+  } else if (podcasts_source->priv->notify_changes) {
+    grl_media_source_notify_change (GRL_MEDIA_SOURCE (podcasts_source),
+                                    NULL,
+                                    GRL_CONTENT_REMOVED,
+                                    TRUE);
   }
 }
 
 static void
-store_podcast (sqlite3 *db, GrlMedia *podcast, GError **error)
+store_podcast (GrlPodcastsSource *podcasts_source,
+               GrlMedia *podcast,
+               GError **error)
 {
   gint r;
   sqlite3_stmt *sql_stmt = NULL;
@@ -779,16 +822,17 @@ store_podcast (sqlite3 *db, GrlMedia *podcast, GError **error)
   desc = grl_media_get_description (podcast);
 
   GRL_DEBUG ("%s", GRL_SQL_STORE_PODCAST);
-  r = sqlite3_prepare_v2 (db,
+  r = sqlite3_prepare_v2 (podcasts_source->priv->db,
 			  GRL_SQL_STORE_PODCAST,
 			  strlen (GRL_SQL_STORE_PODCAST),
 			  &sql_stmt, NULL);
   if (r != SQLITE_OK) {
     GRL_WARNING ("Failed to store podcast '%s': %s", title,
-                 sqlite3_errmsg (db));
-    *error = g_error_new (GRL_CORE_ERROR,
-			  GRL_CORE_ERROR_STORE_FAILED,
-			  "Failed to store podcast '%s'", title);
+                 sqlite3_errmsg (podcasts_source->priv->db));
+    g_set_error (error,
+                 GRL_CORE_ERROR,
+                 GRL_CORE_ERROR_STORE_FAILED,
+                 "Failed to store podcast '%s'", title);
     return;
   }
 
@@ -804,19 +848,28 @@ store_podcast (sqlite3 *db, GrlMedia *podcast, GError **error)
 
   if (r != SQLITE_DONE) {
     GRL_WARNING ("Failed to store podcast '%s': %s", title,
-                 sqlite3_errmsg (db));
-    *error = g_error_new (GRL_CORE_ERROR,
-			  GRL_CORE_ERROR_STORE_FAILED,
-			  "Failed to store podcast '%s'", title);
+                 sqlite3_errmsg (podcasts_source->priv->db));
+    g_set_error (error,
+                 GRL_CORE_ERROR,
+                 GRL_CORE_ERROR_STORE_FAILED,
+                 "Failed to store podcast '%s'", title);
     sqlite3_finalize (sql_stmt);
     return;
   }
 
   sqlite3_finalize (sql_stmt);
 
-  id = g_strdup_printf ("%llu", sqlite3_last_insert_rowid (db));
+  id = g_strdup_printf ("%llu",
+                        sqlite3_last_insert_rowid (podcasts_source->priv->db));
   grl_media_set_id (podcast, id);
   g_free (id);
+
+  if (podcasts_source->priv->notify_changes) {
+    grl_media_source_notify_change (GRL_MEDIA_SOURCE (podcasts_source),
+                                    NULL,
+                                    GRL_CONTENT_ADDED,
+                                    FALSE);
+  }
 }
 
 static void
@@ -943,7 +996,7 @@ parse_entry_idle (gpointer user_data)
       /* Hack: if we emit the last result now the UI may request more results
 	 right away while we are still parsing the XML, so we keep the last
 	 result until we are done processing the whole feed, this way when
-	 the next query arrives all the stuff is stored in the database 
+	 the next query arrives all the stuff is stored in the database
 	 and the query can be resolved normally */
       if (remaining > 0) {
 	osp->os->callback (osp->os->source,
@@ -975,20 +1028,30 @@ parse_entry_idle (gpointer user_data)
 		       0,
 		       osp->os->user_data,
 		       NULL);
-    
+    /* Notify about changes */
+    if (GRL_PODCASTS_SOURCE (osp->os->source)->priv->notify_changes) {
+      media = grl_media_box_new ();
+      grl_media_set_id (media, osp->os->media_id);
+      grl_media_source_notify_change (osp->os->source,
+                                      media,
+                                      GRL_CONTENT_CHANGED,
+                                      FALSE);
+      g_object_unref (media);
+    }
     g_slice_free (OperationSpec, osp->os);
     xmlXPathFreeObject (osp->xpathObj);
     xmlXPathFreeContext (osp->xpathCtx);
     xmlFreeDoc (osp->doc);
     g_slice_free (OperationSpecParse, osp);
   }
-  
+
   return osp->parse_index < osp->parse_count;
 }
 
 static void
 parse_feed (OperationSpec *os, const gchar *str, GError **error)
 {
+  GrlMedia *podcast = NULL;
   xmlDocPtr doc = NULL;
   xmlXPathContextPtr xpathCtx = NULL;
   xmlXPathObjectPtr xpathObj = NULL;
@@ -1040,6 +1103,15 @@ parse_feed (OperationSpec *os, const gchar *str, GError **error)
   GRL_DEBUG ("Got %d streams", stream_count);
   
   if (stream_count <= 0) {
+    if (GRL_PODCASTS_SOURCE (os->source)->priv->notify_changes) {
+      podcast = grl_media_box_new ();
+      grl_media_set_id (podcast, os->media_id);
+      grl_media_source_notify_change (os->source,
+                                      podcast,
+                                      GRL_CONTENT_CHANGED,
+                                      FALSE);
+      g_object_unref (podcast);
+    }
     os->callback (os->source,
 		  os->operation_id,
 		  NULL,
@@ -1190,17 +1262,13 @@ produce_podcasts (OperationSpec *os)
 
   db = GRL_PODCASTS_SOURCE (os->source)->priv->db;
 
-  if (!os->text) {
-    /* Browse */
-    sql = g_strdup_printf (GRL_SQL_GET_PODCASTS, os->count, os->skip);
-  } else if (os->is_query) {
+  if (os->is_query) {
     /* Query */
     sql = g_strdup_printf (GRL_SQL_GET_PODCASTS_BY_QUERY,
 			   os->text, os->count, os->skip);
   } else {
-    /* Search */
-    sql = g_strdup_printf (GRL_SQL_GET_PODCASTS_BY_TEXT,
-			   os->text, os->text, os->count, os->skip);
+    /* Browse */
+    sql = g_strdup_printf (GRL_SQL_GET_PODCASTS, os->count, os->skip);
   }
   GRL_DEBUG ("%s", sql);
   r = sqlite3_prepare_v2 (db, sql, strlen (sql), &sql_stmt, NULL);
@@ -1437,8 +1505,9 @@ grl_podcasts_source_search (GrlMediaSource *source,
   os->skip = ss->skip;
   os->callback = ss->callback;
   os->user_data = ss->user_data;
+  os->is_query = TRUE;
   os->error_code = GRL_CORE_ERROR_SEARCH_FAILED;
-  produce_podcasts (os);
+  produce_podcast_contents_from_db (os);
   g_slice_free (OperationSpec, os);
 }
 
@@ -1515,7 +1584,7 @@ grl_podcasts_source_store (GrlMediaSource *source, GrlMediaSourceStoreSpec *ss)
 			 GRL_CORE_ERROR_STORE_FAILED,
 			 "Cannot create containers. Only feeds are accepted.");
   } else {
-    store_podcast (GRL_PODCASTS_SOURCE (ss->source)->priv->db,
+    store_podcast (GRL_PODCASTS_SOURCE (ss->source),
 		   ss->media,
 		   &error);
   }
@@ -1532,10 +1601,10 @@ grl_podcasts_source_remove (GrlMediaSource *source,
   GRL_DEBUG ("grl_podcasts_source_remove");
   GError *error = NULL;
   if (media_id_is_podcast (rs->media_id)) {
-    remove_podcast (GRL_PODCASTS_SOURCE (rs->source)->priv->db,
+    remove_podcast (GRL_PODCASTS_SOURCE (rs->source),
 		    rs->media_id, &error);
   } else {
-    remove_stream (GRL_PODCASTS_SOURCE (rs->source)->priv->db,
+    remove_stream (GRL_PODCASTS_SOURCE (rs->source),
 		   rs->media_id, &error);
   }
   rs->callback (rs->source, rs->media, rs->user_data, error);
@@ -1543,3 +1612,25 @@ grl_podcasts_source_remove (GrlMediaSource *source,
     g_error_free (error);
   }
 }
+
+static gboolean
+grl_podcasts_source_notify_change_start (GrlMediaSource *source,
+                                         GError **error)
+{
+  GrlPodcastsSource *podcasts_source = GRL_PODCASTS_SOURCE (source);
+
+  podcasts_source->priv->notify_changes = TRUE;
+
+  return TRUE;
+}
+
+static gboolean
+grl_podcasts_source_notify_change_stop (GrlMediaSource *source,
+                                        GError **error)
+{
+  GrlPodcastsSource *podcasts_source = GRL_PODCASTS_SOURCE (source);
+
+  podcasts_source->priv->notify_changes = FALSE;
+
+  return TRUE;
+}
